/*         ______   ___    ___ 
 *        /\  _  \ /\_ \  /\_ \ 
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Forced graphics mode driver (for laptops, etc).
 *
 *      By Laurence Withers.
 *
 *      See readme.txt for copyright information.
 */


#include "allegro.h"
#include "allegro/aintern.h"



static BITMAP* force_graphics_mode(int card, int w, int h, int vw, int vh, int depth)
{
    _DRIVER_INFO* driver_list = _gfx_driver_list;
    int c = 0;
    BITMAP* b = 0;

    if(system_driver->gfx_drivers) driver_list = system_driver->gfx_drivers();
    usetc(allegro_error, 0);

    for(c = 0; driver_list[c].driver; c++) 
        if(driver_list[c].id == card) break;

    if(driver_list[c].driver == 0) {
        do_uconvert("Cannot find card for forced mode; check allegro.cfg", U_ASCII,
            allegro_error, U_CURRENT, 256);
        return 0;
    }

    gfx_driver = driver_list[c].driver;
    gfx_driver->name = gfx_driver->desc = get_config_text(gfx_driver->ascii_name);

    set_color_depth(depth);

    b = gfx_driver->init(w, h, vw, vh, depth);

    return b;
}



static BITMAP* force_init(int w, int h, int v_w, int v_h, int color_depth)
{
    int count = 0, i = 0;
    char variable_name[250], tmp[250], forced_graphics[250];
    int card = 0;

    int max_w = 0, max_h = 0, req_w = 0, req_h = 0, req_vw = 0, req_vh = 0;

    int best_card = 0, best_w = 0, best_h = 0;
    int new_depth = 0, best_depth = 0;

    /* Search for nearest mode. It is OK to go larger, but not OK to go
     * smaller. Also, `color_depth' must match one of the listed depths.
     */

    do_uconvert("ForcedGraphics", U_ASCII, forced_graphics, U_CURRENT, 250);

    usprintf(variable_name, uconvert_ascii("%d_Driver", tmp), count);
    card = get_config_id(forced_graphics, variable_name, 0);

    while(card) {

        /* We don't really need a while loop, but it helps to be able to
         * use `break'.
         */
        while (card != GFX_FORCE) {

            /* Read the maximum width and height that this setup is good
             * for. If it is not good enough, then exit.
             */

            usprintf(variable_name, uconvert_ascii("%d_MaxWidth", tmp), count);
            max_w = get_config_int(forced_graphics, variable_name, 0);
            if(max_w < w) break;

            usprintf(variable_name, uconvert_ascii("%d_MaxHeight", tmp), count);
            max_h = get_config_int(forced_graphics, variable_name, 0);
            if(max_h < h) break;

            /* Check if color depth is allowed, and for translation */

            usprintf(variable_name, uconvert_ascii("%d_ColorDepth_%d", tmp), count, color_depth);
            new_depth = get_config_int(forced_graphics, variable_name, 0);

            if(!new_depth) break;

            /* Now compare this with our best driver's info. */

            if(best_w != 0)
                if( (max_w - best_w) + (max_h - best_h) > 0) break;

            /* Use this as our best driver -- retrieve new stuff from
             * config file, save driver, etc. */

            best_card = card;
            best_depth = new_depth;

            usprintf(variable_name, uconvert_ascii("%d_ReqWidth", tmp), count);
            req_w = get_config_int(forced_graphics, variable_name, 0);
            usprintf(variable_name, uconvert_ascii("%d_ReqVWidth", tmp), count);
            req_vw = get_config_int(forced_graphics, variable_name, 0);
            usprintf(variable_name, uconvert_ascii("%d_ReqHeight", tmp), count);
            req_h = get_config_int(forced_graphics, variable_name, 0);
            usprintf(variable_name, uconvert_ascii("%d_ReqVHeight", tmp), count);
            req_vh = get_config_int(forced_graphics, variable_name, 0);

            best_w = req_w;
            best_h = req_h;

            break;
        }

        count++;
        usprintf(variable_name, uconvert_ascii("%d_Driver", tmp), count);
        card = get_config_id(forced_graphics, variable_name, 0);
    }

    /* Now that we have read the config file, we should have the
     * necessary parameters for setting the best mode. However, we don't
     * necessarily have a card; if this is the case, then we must set
     * an error message.
     */

    if(best_card) {
        return force_graphics_mode(best_card, req_w, req_h, req_vw, req_vh, best_depth);
    }

    ustrcpy(allegro_error, get_config_text("Unable to find a suitable forced graphics driver"));
    return 0;
}



/* This is our pseudo-driver structure. We only need to set a couple of
 * things in the structure, since they are all that is used. Once the
 * force_init() function is called, the graphics driver is changed
 * anyway.
 */
GFX_DRIVER gfx_force = {

    GFX_FORCE,
    empty_string,
    empty_string,
    "Forced driver",
    force_init,

    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0
};


